import logging
from typing import *

import qcanvas_backend.database.types as db
from qcanvas_backend.net.resources.download.resource_manager import ResourceManager
from qcanvas_backend.net.sync.sync_receipt import SyncReceipt

from qcanvas.ui.course_viewer.tabs.file_tab.file_tree import FileTree
from qcanvas.ui.memory_tree import MemoryTreeWidgetItem

_logger = logging.getLogger(__name__)


class PagesFileTree(FileTree):

    def __init__(self, tree_name: str, *, resource_manager: ResourceManager):
        super().__init__(
            tree_name=f"{tree_name}.pages", resource_manager=resource_manager
        )

    def create_tree_items(
        self, data: db.Course, sync_receipt: SyncReceipt
    ) -> Sequence[MemoryTreeWidgetItem]:
        widgets = []

        for group in data.modules:  # type: db.Module
            if len(group.content_items) == 0:
                continue

            group_widget = self._create_group_widget(group, sync_receipt)
            items_in_group = set()

            for item in group.content_items:
                for resource in item.resources:  # type: db.Resource

                    if resource.id not in items_in_group:
                        items_in_group.add(resource.id)
                    else:
                        continue

                    resource_widget = self._create_resource_widget(
                        resource, sync_receipt
                    )

                    group_widget.addChild(resource_widget)

            if group_widget.childCount() > 0:
                widgets.append(group_widget)

        return widgets



import logging

from qtpy.QtCore import QFileInfo, QMimeDatabase
from qtpy.QtGui import QIcon
from qtpy.QtWidgets import QApplication, QFileIconProvider, QStyle

import qcanvas.util.runtime as runtime

_logger = logging.getLogger(__name__)

# Windows and linux have different ways of doing this
if runtime.is_running_on_windows:
    _icon_provider = QFileIconProvider()

    def icon_for_filename(file_name: str) -> QIcon:
        return _icon_provider.icon(QFileInfo(file_name))

else:
    _mime_database = QMimeDatabase()
    _default_icon = None

    def icon_for_filename(file_name: str) -> QIcon:
        global _default_icon

        for mime_type in _mime_database.mimeTypesForFileName(file_name):
            icon = QIcon.fromTheme(mime_type.iconName())

            if not icon.isNull():
                return icon

        if _default_icon is None:
            _default_icon = QApplication.style().standardIcon(
                QStyle.StandardPixmap.SP_FileIcon
            )

        return _default_icon



import logging

import darkdetect
import qdarktheme
from qtpy.QtGui import QGuiApplication
from qtpy.QtWidgets import QApplication, QStyleFactory

_logger = logging.getLogger(__name__)

default_theme = "auto"


def ensure_theme_is_valid(theme: str) -> str:
    if theme not in ["auto", "light", "dark", "native"]:
        return default_theme
    else:
        return theme


def apply(theme: str) -> None:
    theme = ensure_theme_is_valid(theme)

    if theme != "native":

        qdarktheme.setup_theme(
            theme,
            custom_colors={"primary": "e02424"},
        )

        QApplication.setStyle(QStyleFactory.create("Fusion"))

        _logger.debug("darkdetect says: %s", darkdetect.theme())
        _logger.debug("qt says: %s", QGuiApplication.styleHints().colorScheme())



import logging

import qcanvas_backend.database.types as db
from qcanvas_backend.net.resources.download.resource_manager import ResourceManager
from qcanvas_backend.net.sync.sync_receipt import SyncReceipt
from qtpy.QtCore import Slot
from qtpy.QtWidgets import *

from qcanvas.ui.course_viewer.tabs.assignment_tab import AssignmentTab
from qcanvas.ui.course_viewer.tabs.file_tab import FileTab
from qcanvas.ui.course_viewer.tabs.mail_tab import MailTab
from qcanvas.ui.course_viewer.tabs.page_tab import PageTab
from qcanvas.util.basic_fonts import bold_font
from qcanvas.util.layouts import layout
from qcanvas.util.ui_tools import make_truncatable

_logger = logging.getLogger(__name__)


class CourseViewer(QWidget):
    def __init__(
        self,
        course: db.Course,
        downloader: ResourceManager,
        *,
        sync_receipt: SyncReceipt
    ):
        super().__init__()
        # todo this is a mess. there are several other messes like this too, do they all have to be a mess?
        self._course_id = course.id

        self._course_label = QLabel(course.name)
        self._course_label.setFont(bold_font)
        make_truncatable(self._course_label)

        self._pages_tab = PageTab.create_from_receipt(
            course=course,
            downloader=downloader,
            sync_receipt=sync_receipt,
        )
        self._assignments_tab = AssignmentTab.create_from_receipt(
            course=course,
            downloader=downloader,
            sync_receipt=sync_receipt,
        )
        self._mail_tab = MailTab.create_from_receipt(
            course=course,
            downloader=downloader,
            sync_receipt=sync_receipt,
        )
        self._files_tab = FileTab.create_from_receipt(
            course=course,
            downloader=downloader,
            sync_receipt=sync_receipt,
        )

        self._tabs = QTabWidget()
        self._tabs.addTab(self._files_tab, "Files")
        self._tabs.addTab(self._pages_tab, "Pages")
        self._tabs.addTab(self._assignments_tab, "Assignments")
        self._tabs.addTab(self._mail_tab, "Mail")
        # self._tabs.addTab(QLabel("Not implemented"), "Panopto")  # The meme lives on!

        self.setLayout(layout(QVBoxLayout, self._course_label, self._tabs))

        self._tabs.currentChanged.connect(self._tab_changed)

        self._highlight_tabs(sync_receipt)
        self._unhighlight_tab(0)  # Because the first tab always gets auto-selected

    def reload(self, course: db.Course, *, sync_receipt: SyncReceipt) -> None:
        self._files_tab.reload(course, sync_receipt=sync_receipt)
        self._pages_tab.reload(course, sync_receipt=sync_receipt)
        self._assignments_tab.reload(course, sync_receipt=sync_receipt)
        self._mail_tab.reload(course, sync_receipt=sync_receipt)
        self._highlight_tabs(sync_receipt)

    @Slot(int)
    def _tab_changed(self, index: int) -> None:
        if index != -1:
            self._unhighlight_tab(index)

    def _highlight_tabs(self, sync_receipt: SyncReceipt) -> None:
        updates = sync_receipt.updates_by_course.get(self._course_id, None)

        if updates is not None:
            if len(updates.updated_resources) > 0:
                self._highlight_tab(0)

            if len(updates.updated_pages) > 0:
                self._highlight_tab(1)

            if len(updates.updated_assignments) > 0:
                self._highlight_tab(2)

            if len(updates.updated_messages) > 0:
                self._highlight_tab(3)
        else:
            for index in range(0, 4):
                self._unhighlight_tab(index)

    def _highlight_tab(self, tab_index: int) -> None:
        self._tabs.setTabText(tab_index, "* " + self._tabs.tabText(tab_index))

    def _unhighlight_tab(self, tab_index: int) -> None:
        self._tabs.setTabText(
            tab_index, self._tabs.tabText(tab_index).replace("* ", "")
        )



import logging
from math import floor
from typing import *

import qcanvas_backend.database.types as db
from qcanvas_backend.net.resources.download.resource_manager import ResourceManager
from qcanvas_backend.net.sync.sync_receipt import SyncReceipt, empty_receipt
from qtpy.QtCore import Qt
from qtpy.QtGui import QIcon
from qtpy.QtWidgets import *

from qcanvas import icons
from qcanvas.ui.course_viewer.course_viewer import CourseViewer

_logger = logging.getLogger(__name__)


# todo needs to handle dark mode
class _PlaceholderLogo(QLabel):
    """
    Automatically resizing logo icon for when no course is selected
    """

    def __init__(self):
        super().__init__()
        self._icon = QIcon(icons.logo_transparent_light)
        self._old_width = -1
        self._old_height = -1
        self.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.setSizePolicy(QSizePolicy.Policy.Ignored, QSizePolicy.Policy.Ignored)

    def resizeEvent(self, event) -> None:
        # Calculate the size of the logo as half of the width/height with a max size of 1000x1000
        width = min(floor(self.width() * 0.5), 500)
        height = min(floor(self.height() * 0.5), 500)

        if width == self._old_width or height == self._old_height:
            return
        else:
            self._old_width = width
            self._old_height = height

        self.setPixmap(self._icon.pixmap(width, height))


class CourseViewerContainer(QStackedWidget):
    def __init__(self, downloader: ResourceManager):
        super().__init__()
        self._course_viewers: dict[str, CourseViewer] = {}
        self._downloader = downloader
        self._last_course_id: Optional[str] = None
        self._last_sync_receipt: SyncReceipt = empty_receipt()
        self._placeholder = _PlaceholderLogo()
        self.addWidget(self._placeholder)

    def show_blank(self) -> None:
        self._last_course_id = None
        self.setCurrentWidget(self._placeholder)

    def load_course(self, course: db.Course) -> None:
        if course.id not in self._course_viewers:
            viewer = CourseViewer(
                course=course,
                downloader=self._downloader,
                sync_receipt=self._last_sync_receipt,
            )
            self._course_viewers[course.id] = viewer
            self.addWidget(viewer)
        else:
            viewer = self._course_viewers[course.id]

        self.setCurrentWidget(viewer)
        self._last_course_id = course.id

    async def reload_all(
        self, courses: Sequence[db.Course], *, sync_receipt: SyncReceipt
    ) -> None:
        self._last_sync_receipt = sync_receipt
        for course in courses:
            if course.id in self._course_viewers:
                viewer = self._course_viewers[course.id]
                viewer.reload(course, sync_receipt=sync_receipt)
